function T = TherMod(Parms)
%function T = TherMod(Parms)
%
% The following globals must be defined
% global e Top2 AgeCorr AgeH Conductivity SurfaceTemp SurfaceTempAve SurfaceTempGradient MohoTemp LIDe LO LA
%
% INPUT: Parms structure containing
%   For temperature profile:
%	j,k - vector of locations on NCM grid.
%		OR
%	lat,lon - latitude and longitude of point of interest 
%
%	Depth - depth vector in meters
%
%   For temperature map:
%	Depth - depth value at which map is desired in meters
%		OR
%
%	j,k - grid of locations within NCM grid.
%		OR
%	lat,lon - latitude and longitude grid
%		AND
%	Depth - depth value or grid in meters the same size as j,k or lat,lon
%
%

global e Top2 AgeCorr AgeH Conductivity SurfaceTemp SurfaceTempAve SurfaceTempGradient ...
	MohoTemp LIDe TransDepth TransDepthType TransDepthOcean TransDepthTypeOcean

if isfield(Parms,'j') | isfield(Parms,'lon')
	if isfield(Parms,'lon')
		[j,k] = GetJK(Parms.lat,Parms.lon);
	elseif isfield(Parms,'j')
		j = Parms.j;
		k = Parms.k;
	end
	[x,y] = meshgrid(1:4901,1:3201);
	ei = double(interp2(x,y,e,k,j,'cubic'));
	Ts = double(interp2(x,y,SurfaceTemp,k,j,'cubic'));
	ks = double(interp2(x,y,Conductivity,k,j,'cubic'));
	TsAve = double(interp2(x,y,SurfaceTempAve,k,j,'cubic'));
	dTs = double(interp2(x,y,SurfaceTempGradient,k,j,'cubic'));
	Tm = double(interp2(x,y,MohoTemp,k,j,'cubic'));
	Md = ei - double(interp2(x,y,LIDe,k,j,'cubic'));
	DepthOceanPlate = ei - double(interp2(x,y,Top2,k,j,'cubic'));
	AC = double(interp2(x,y,AgeCorr,k,j,'cubic'));
	AH = double(interp2(x,y,AgeH,k,j,'cubic'));
else
	ei = double(e);
	Ts = double(SurfaceTemp);
	ks = double(Conductivity);
	TsAve = double(SurfaceTempAve);
	dTs = double(SurfaceTempGradient);
	Tm = double(MohoTemp);
	Md = ei - double(LIDe);
	DepthOceanPlate = ei - double(Top2);
	AC = double(AgeCorr);
	AH = double(AgeH);
end
Depth = Parms.Depth;

[ndj,ndk] = size(Depth);
if ndj == 1 | ndk == 1
	if ndj > ndk
		Depth = Depth';
	end
end
if ndj*ndk == 1 | (ndj > 1 & ndk > 1)
	l = find(TransDepthType == 3);
	TransDepthType(l) = 2;
	l = find(TransDepthTypeOcean == 3);
	TransDepthTypeOcean(l) = 2;
	if ndj*ndk == 1
		if exist('j','var')
        		Depth = Depth*ones(size(j));
		else
        		Depth = Depth*ones(size(e));
		end
	end
	makeMap = 1;
else
	makeMap = 0;
end

hr = 0.3*Md;
km = 3.6;
kum = 4.5; % Stacey (1992), p 287
kr = Md./log(km./ks);

secPerYear = 365.24*24*60*60;

LithMantleGrad2 = 0.0004;	    % From Stacey (2008), Appendix G3 from ~200 to ~400 km depth
TMantle = LithMantleGrad2*(Depth - 410e3) + 1579.85;

TM = 1300-273.15; % Turcotte and Schubert, p. 164
kappa = 1e-6;
AgeSeconds = AC*1e6*secPerYear;

% Correction for higher surface temperature of DTS over the last 10 ka
P = 10e3*secPerYear;
K = 1e-6;
CharLength = sqrt(K*P/pi);
DTS = Ts - TsAve;

% Correct for approaching the solidus
% Basalt solidus from Turcotte and Schubert, Figure 4-8
SolidusGradient = 200/100e3;
TSolidus = 1100 + Depth*SolidusGradient;

TMS = 1./((1./TMantle).^30 + (1./TSolidus).^30).^(1/30) + 273.15;

% With increasing conductivity and decreasing heat production
Hr = 1./(1./hr + 1./kr);
pHs_o_k = (Tm.*(1 + Md./kr) - dTs.*Md -  TsAve.*(1 + Md./kr))./(Hr.^2 - Hr.*Md - Hr.^2.*exp(-Md./Hr));
c1 = dTs + TsAve./kr - pHs_o_k.*Hr;
c2 = TsAve + pHs_o_k.*Hr.^2;

if makeMap == 1
	if isfield(Parms,'j') | isfield(Parms,'lon')
		trd = double(interp2(x,y,TransDepth,k,j,'cubic'));
		trdt = double(interp2(x,y,TransDepthType,k,j,'nearest'));
		trdo = double(interp2(x,y,TransDepthOcean,k,j,'cubic'));
		trdto = double(interp2(x,y,TransDepthTypeOcean,k,j,'nearest'));
	else
		trd = double(TransDepth);
		trdt = double(TransDepthType);
		trdo = double(TransDepthOcean);
		trdto = double(TransDepthTypeOcean);
	end

	T1 = (c1.*Depth + c2 - pHs_o_k.*Hr.^2.*exp(-Depth./Hr))./(1 + Depth./kr);

	% Correct T1 for very low gradients
	l = find(trdt == 2 & Depth > trd);
	Ttr = (c1(l).*trd(l) + c2(l) - pHs_o_k(l).*Hr(l).^2.*exp(-trd(l)./Hr(l)))./ ...
		(1 + trd(l)./kr(l));
	T1(l) = Ttr + 2.0*SolidusGradient*(Depth(l) - trd(l));

	m = find(Depth(l) > Md(l));
	J = l(m);
	dTdZmSub = km*2.0*SolidusGradient/kum;
	Ttr = (c1(J).*trd(J) + c2(J) - pHs_o_k(J).*Hr(J).^2.*exp(-trd(J)./Hr(J)))./ ...
		(1 + trd(J)./kr(J));
	Tm(J) = Ttr + 2.0*SolidusGradient*(Md(J) - trd(J));
	T1(J) = dTdZmSub.*(Depth(J) - Md(J)) + Tm(J);

	% Modify normal geotherm for depth greater than Moho, Md
	l = find(trdt == 0 & Depth > Md);
	dTdZm = -(1./kr(l)).*(c1(l).*Md(l) + c2(l) - pHs_o_k(l).*Hr(l).^2.* ...
		exp(-Md(l)./Hr(l)))./(1 + Md(l)./kr(l)).^2 + ...
        	(c1(l) + pHs_o_k(l).*Hr(l).*exp(-Md(l)./Hr(l)))./(1 + Md(l)./kr(l));
	dTdZmSub = km*dTdZm/kum;
	T1(l) = dTdZmSub.*(Depth(l) - Md(l)) + Tm(l);

	% add to TMS
	T1 = T1 + 273.15;
	T1 = 1./((1./T1).^10 + (1./TMS).^10).^(1/10);
	T1 = T1 - 273.15;
	T2 = T1;

	l = find(Depth > DepthOceanPlate);
	TsO = zeros(size(T1));
	if length(l) > 0
		if isfield(Parms,'j') | isfield(Parms,'lon')
			TParms.j = j;
			TParms.k = k;
		end
		TParms.Depth = DepthOceanPlate;
		TsO = TherMod(TParms);
		phi = 2*sqrt(kappa*AgeSeconds(l));
		dt = (max(TM*ones(size(TsO(l))),TsO(l)) - TsO(l));
		T2(l) = dt.*erf((Depth(l) - DepthOceanPlate(l))./phi) + TsO(l);
		m = find(Depth(l) > trdo(l) & (trdto(l) == 1 | trdto(l) == 2));
		if length(m) > 0
			K = find(trdto(l(m)) == 1);
			n = l(m(K));
			z = (trdo(n) - DepthOceanPlate(n))./phi(m(K));
			dTdZ = dt(m(K))*(2/sqrt(pi)).*exp(-z.^2).*(1./phi(m(K)));
			Ttr = dt(m(K)).*erf(z) + TsO(n);
			T2(n) = Ttr + dTdZ.*(Depth(n) - trdo(n));

			K = find(trdto(l(m)) == 2);
			n = l(m(K));
			Ttr = dt(m(K)).*erf((trdo(n) - DepthOceanPlate(n))./phi(m(K))) + TsO(n);
			T2(n) = Ttr + LithMantleGrad2*(Depth(n) - trdo(n));
		end
		T2 = T2 + 273.15;
		T2 = 1./((1./T2).^10 + (1./TMS).^10).^(1/10);
		T2 = T2 - 273.15;
	end
	T = double(AH).*T2 + (1-double(AH)).*T1;
	DT = DTS.*exp(-Depth/CharLength);
	T = T + DT;
	return;
end

nLocs = length(j);
for l = 1:nLocs
	% With increasing conductivity and decreasing heat production
	T1 = (c1(l).*Depth + c2(l) - pHs_o_k(l)*Hr(l)^2*exp(-Depth/Hr(l)))./(1 + Depth/kr(l));
	dTdZ = gradient(T1)/(Depth(2)-Depth(1));
	j = find(dTdZ < 2.0*SolidusGradient);
	if length(j) > 0
		if j(1) > 1
			J = j(1)-1:length(Depth);
			T1(J) = T1(j(1)-1) + 2.0*SolidusGradient*(Depth(J) - Depth(j(1)-1));;
		end
	end

	% Steepen gradient below the Moho
	if length(j) == 0
		dTdZm = -(1/kr(l))*(c1(l)*Md(l) + c2(l) - ...
			pHs_o_k(l)*Hr(l)^2*exp(-Md(l)/Hr(l)))/(1 + Md(l)/kr(l))^2 + ...
			(c1(l) + pHs_o_k(l)*Hr(l)*exp(-Md(l)/Hr(l)))/(1 + Md(l)/kr(l));
		dTdZmSub = km*dTdZm/kum;
		j = find(Depth >= Md(l));
		T1(j) = Tm(l) + dTdZmSub.*(Depth(j) - Md(l));
	else
		if Depth(j(1)) > Md(l)
			dTdZm = -(1/kr(l))*(c1(l)*Md(l) + c2(l) - ...
				pHs_o_k(l)*Hr(l)^2*exp(-Md(l)/Hr(l)))/(1 + Md(l)/kr(l))^2 + ...
				(c1(l) + pHs_o_k(l)*Hr(l)*exp(-Md(l)/Hr(l)))/(1 + Md(l)/kr(l));
			dTdZmSub = km*dTdZm/kum;

			j = find(Depth >= Md(l));
			T1(j) = Tm(l) + dTdZmSub.*(Depth(j) - Md(l));
		else
			dTdZmSub = km*2.0*SolidusGradient/kum;
			j = find(Depth >= Md(l));
			Tm(l) = interp1(Depth,T1,Md(l),'linear');
			T1(j) = Tm(l) + dTdZmSub.*(Depth(j) - Md(l));
		end
	end
	T1 = T1 + 273.15;
	T1 = 1./((1./T1).^10 + (1./TMS).^10).^(1/10);
	T1 = T1 - 273.15;
	T2 = T1;
	
	j = find(Depth > DepthOceanPlate(l));
	if length(j) > 0
		TsO = T1(j(1));
        	T2(j) = (max(TM,TsO) - TsO).*erf((Depth(j) - DepthOceanPlate(l))./(2*sqrt(kappa*AgeSeconds(l)))) + TsO;
		dTdD = gradient(T2)./gradient(Depth);
		k = find(T2 > TMS - 273.15);
		if length(k) > 0
			K = k(1):length(Depth);
			T2(K) = T2(k(1)) + dTdD(k(1))*(Depth(K) - Depth(k(1)));
		else
			k = find(dTdD < LithMantleGrad2 & Depth > DepthOceanPlate(l));
			if length(k) > 0
				if k(1) > 1
					K = k(1)-1:length(Depth);
					T2(K) = T2(k(1)-1) + LithMantleGrad2*(Depth(K) - Depth(k(1)-1));
				else
					K = k(1):length(Depth);
					T2(K) = T2(k(1)) + LithMantleGrad2*(Depth(K) - Depth(k(1)));
				end
			end
		end
	end
	
	T(l,:) = AH(l).*T2 + (1-AH(l)).*T1;

	% Correct for higher surface temperature of DTS over the last 100 ka
	DT = DTS(l)*exp(-Depth/CharLength);
	T(l,:) = T(l,:) + DT;
end
